#!/usr/bin/env python3
import os
import sys


def cli():
    os.environ["DJANGO_SETTINGS_MODULE"] = "reversion_compare_project.settings"
    print(f"\nUse DJANGO_SETTINGS_MODULE={os.environ['DJANGO_SETTINGS_MODULE']!r}")
    from django.core.management import execute_from_command_line

    print(' '.join(sys.argv))
    execute_from_command_line(sys.argv)


def start_test_server():
    sys.argv = [__file__, "run_testserver"]
    cli()


if __name__ == "__main__":
    cli()



from reversion_compare.views import HistoryCompareDetailView
from reversion_compare_project.models import SimpleModel


class SimpleModelHistoryCompareView(HistoryCompareDetailView):
    model = SimpleModel



#!/usr/bin/env python

"""
    django-reversion-compare unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    I used the setup from reversion_compare_test_project !

    TODO:
        * models.IntegerField()

    :copyleft: 2012-2020 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from reversion.models import Version

from reversion_compare_project.utils.fixtures import Fixtures
from reversion_compare_project.utils.test_cases import BaseTestCase


class OneToOneFieldTest(BaseTestCase):
    "Test a model which uses a custom reversion manager."

    def setUp(self):
        super().setUp()
        fixtures = Fixtures(verbose=False)
        self.person, self.item = fixtures.create_PersonIdentity_data()

        queryset = Version.objects.get_for_object(self.person)
        self.version_ids = queryset.values_list("pk", flat=True)

    def test_select_compare(self):
        response = self.client.get(f"/en/admin/reversion_compare_project/person/{self.person.pk}/history/")

        self.assertContainsHtml(
            response,
            '<input type="submit" value="compare">',
            f'<input type="radio" name="version_id1" value="{self.version_ids[0]:d}" style="visibility:hidden" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids[0]:d}" checked="checked" />',
            f'<input type="radio" name="version_id1" value="{self.version_ids[1]:d}" checked="checked" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids[1]:d}" />',
        )

    def test_compare(self):
        response = self.client.get(
            f"/en/admin/reversion_compare_project/person/{self.person.pk}/history/compare/",
            data={"version_id2": self.version_ids[0], "version_id1": self.version_ids[1]},
        )

        self.assertContainsHtml(
            response,
            """
            <div class="module">
                <pre class="highlight">
                    <del>- Dave</del>
                    <ins>+ John</ins>
                </pre>
            </div>
            """,
            """
            <p class="highlight">
                <i>add:</i> <ins>None</ins>
            </p>
            """,
            "<blockquote>version 2: change person name.</blockquote>",
        )



#!/usr/bin/env python

"""
    django-reversion-compare unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    I used the setup from reversion_compare_test_project !

    TODO:
        * models.OneToOneField()
        * models.IntegerField()

    :copyleft: 2012-2020 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from reversion import is_registered
from reversion.models import Revision, Version

from reversion_compare_project.models import SimpleModel
from reversion_compare_project.utils.fixtures import Fixtures
from reversion_compare_project.utils.test_cases import BaseTestCase


class SimpleModelTest(BaseTestCase):
    """
    unittests that used reversion_compare_test_app.models.SimpleModel

    Tests for the basic functions.
    """

    def setUp(self):
        super().setUp()
        fixtures = Fixtures(verbose=False)
        # fixtures = Fixtures(verbose=True)
        self.item1, self.item2 = fixtures.create_Simple_data()

        queryset = Version.objects.get_for_object(self.item1)
        self.version_ids1 = queryset.values_list("pk", flat=True)

        queryset = Version.objects.get_for_object(self.item2)
        self.version_ids2 = queryset.values_list("pk", flat=True)

    def test_initial_state(self):
        self.assertTrue(is_registered(SimpleModel))

        self.assertEqual(SimpleModel.objects.count(), 2)
        self.assertEqual(SimpleModel.objects.all()[0].text, "version two")

        self.assertEqual(Version.objects.get_for_object(self.item1).count(), 2)
        self.assertEqual(Version.objects.get_for_object(self.item2).count(), 5)
        self.assertEqual(Revision.objects.all().count(), 7)
        self.assertEqual(Version.objects.all().count(), 7)

        # query.ValuesListQuerySet() -> list():
        self.assertEqual(list(self.version_ids1), [2, 1])
        self.assertEqual(list(self.version_ids2), [7, 6, 5, 4, 3])

    def test_select_compare1(self):
        response = self.client.get(f"/en/admin/reversion_compare_project/simplemodel/{self.item1.pk}/history/")
        #  debug_response(response) # from django-tools
        self.assertContainsHtml(
            response,
            '<input type="submit" value="compare">',
            f'<input type="radio" name="version_id1" value="{self.version_ids1[0]:d}" style="visibility:hidden" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids1[0]:d}" checked="checked" />',
            f'<input type="radio" name="version_id1" value="{self.version_ids1[1]:d}" checked="checked" />',
            f'<input type="radio" name="version_id2" value="{self.version_ids1[1]:d}" />',
        )

    def test_select_compare2(self):
        response = self.client.get(f"/en/admin/reversion_compare_project/simplemodel/{self.item2.pk}/history/")
        # debug_response(response) # from django-tools
        for i in range(4):
            if i == 0:
                comment = f"create v{i:d}"
            else:
                comment = f"change to v{i:d}"

            self.assertContainsHtml(response, f"<td>{comment}</td>", '<input type="submit" value="compare">')

    def test_diff(self):
        response = self.client.get(
            f"/en/admin/reversion_compare_project/simplemodel/{self.item1.pk}/history/compare/",
            data={"version_id2": self.version_ids1[0], "version_id1": self.version_ids1[1]},
        )
        # debug_response(response) # from django-tools
        self.assertContainsHtml(
            response,
            """
            <div class="module">
                <pre class="highlight">
                    <del>- version one</del>
                    ?          --
                    <ins>+ version two</ins>
                    ?         ++
                </pre>
            </div>
            """,
            "<blockquote>simply change the CharField text.</blockquote>",  # edit comment
        )

    def test_prev_next_buttons(self):
        base_url = f"/en/admin/reversion_compare_project/simplemodel/{self.item2.pk}/history/compare/"
        for i in range(4):
            # IDs: 3,4,5,6
            id1 = i + 3
            id2 = i + 4
            response = self.client.get(base_url, data={"version_id2": id2, "version_id1": id1})
            self.assertContainsHtml(
                response,
                f"<del>- v{i:d}</del>",
                "<ins>+ v%i</ins>" % (i + 1),
                "<blockquote>change to v%i</blockquote>" % (i + 1),
            )
            # print("\n\n+++", i)
            # for line in response.content.decode("utf-8").split("\n"):
            #     if "next" in line or "previous" in line:
            #         print(line)
            """
            +++ 0
                <li><a href="?version_id1=4&amp;version_id2=5">next &rsaquo;</a></li>
            +++ 1
                <li><a href="?version_id1=3&amp;version_id2=4">&lsaquo; previous</a></li>
                <li><a href="?version_id1=5&amp;version_id2=6">next &rsaquo;</a></li>
            +++ 2
                <li><a href="?version_id1=4&amp;version_id2=5">&lsaquo; previous</a></li>
                <li><a href="?version_id1=6&amp;version_id2=7">next &rsaquo;</a></li>
            +++ 3
                <li><a href="?version_id1=5&amp;version_id2=6">&lsaquo; previous</a></li>
            """

            next = f'<a href="?version_id1={i + 4}&amp;version_id2={i + 5}">next &rsaquo;</a>'
            prev = f'<a href="?version_id1={i + 2}&amp;version_id2={i + 3}">&lsaquo; previous</a>'

            if i == 0:
                self.assertNotContains(response, "previous")
                self.assertContains(response, "next")
                self.assertContainsHtml(response, next)
            elif i == 3:
                self.assertContains(response, "previous")
                self.assertNotContains(response, "next")
                self.assertContainsHtml(response, prev)
            else:
                self.assertContains(response, "previous")
                self.assertContains(response, "next")
                self.assertContainsHtml(response, prev, next)



"""
    admin
    ~~~~~

    All example admin classes would be used in django-reversion-compare unittests, too.

    :copyleft: 2012-2015 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from django.contrib import admin

from reversion_compare.admin import CompareVersionAdmin
from reversion_compare_project.models import (
    Car,
    CountryFieldTestModel,
    CustomModel,
    Factory,
    Identity,
    MigrationModel,
    Person,
    Pet,
    SimpleModel,
    TemplateField,
    VariantModel,
)


class SimpleModelAdmin(CompareVersionAdmin):
    pass


admin.site.register(SimpleModel, SimpleModelAdmin)


class MigrationModelAdmin(CompareVersionAdmin):
    pass


admin.site.register(MigrationModel, MigrationModelAdmin)


class FactoryAdmin(CompareVersionAdmin):
    pass


admin.site.register(Factory, FactoryAdmin)


class CarAdmin(CompareVersionAdmin):
    pass


admin.site.register(Car, CarAdmin)


class PersonAdmin(CompareVersionAdmin):
    pass


admin.site.register(Person, PersonAdmin)


class PetAdmin(CompareVersionAdmin):
    pass


admin.site.register(Pet, PetAdmin)


class VariantModelAdmin(CompareVersionAdmin):
    pass


admin.site.register(VariantModel, VariantModelAdmin)


class CustomModelAdmin(CompareVersionAdmin):
    pass


admin.site.register(CustomModel, CustomModelAdmin)

admin.site.register(Identity, CustomModelAdmin)


class TemplateFieldModelAdmin(CompareVersionAdmin):
    pass


admin.site.register(TemplateField, TemplateFieldModelAdmin)


@admin.register(CountryFieldTestModel)
class CountryFieldTestModelAdmin(CompareVersionAdmin):
    pass


"""
class RelatedModelInline(admin.StackedInline):
    model = RelatedModel


class GenericRelatedInline(GenericStackedInline):
    model = GenericRelatedModel


class ChildModelAdmin(CompareVersionAdmin):
    inlines = RelatedModelInline, GenericRelatedInline,
    list_display = ("parent_name", "child_name",)
    list_editable = ("child_name",)

admin.site.register(ChildModel, ChildModelAdmin)


class FlatExampleModelAdmin(CompareVersionAdmin):
    def compare_sub_text(self, obj, version1, version2, value1, value2):
        ''' field_name example '''
        return "%s -> %s" % (value1, value2)

admin.site.register(FlatExampleModel, FlatExampleModelAdmin)


class HobbyModelAdmin(CompareVersionAdmin):
    pass
admin.site.register(HobbyModel, HobbyModelAdmin)

class PersonModelAdmin(CompareVersionAdmin):
    pass
admin.site.register(PersonModel, PersonModelAdmin)

class GroupModelAdmin(CompareVersionAdmin):
    pass
admin.site.register(GroupModel, GroupModelAdmin)

class MembershipModelAdmin(CompareVersionAdmin):
    pass
admin.site.register(MembershipModel, MembershipModelAdmin)
"""



"""
    Helper to compare raw version data.
    see: https://github.com/etianen/django-reversion/issues/859

    :copyleft: 2021 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


import json
import pprint

from django.core.serializers.json import DjangoJSONEncoder
from reversion.models import Version


def get_version_data(version):
    """
    Get field data from a Version instance.
    """
    assert isinstance(version, Version)
    assert version.format == 'json', f'{version.format!r} not supported (only JSON)'

    json_string = version.serialized_data
    version_data = json.loads(json_string)
    # version_data looks like:
    # [{'fields': {'info': 'Migration state 2 - version 4',
    #              'number_then_text': 111,
    #              'text': 'Now this is a short text!!!'},
    #   'model': 'reversion_compare_project.migrationmodel',
    #   'pk': 1}]
    assert len(version_data) == 1
    fields_data = version_data[0]['fields']
    return fields_data


def pformat(value):
    try:
        return DjangoJSONEncoder(indent=4, sort_keys=True, ensure_ascii=False).encode(value)
    except TypeError:
        # Fallback if values are not serializable with JSON:
        return pprint.pformat(value, width=120)



from diff_match_patch import diff_match_patch

from reversion_compare.helpers import (
    EFFICIENCY,
    SEMANTIC,
    diff2lines,
    generate_dmp_diff,
    generate_ndiff,
    html_diff,
    lines2html,
)


DIFF_EQUAL = diff_match_patch.DIFF_EQUAL
DIFF_INSERT = diff_match_patch.DIFF_INSERT
DIFF_DELETE = diff_match_patch.DIFF_DELETE


def test_generate_ndiff():
    html = generate_ndiff(
        value1='one',
        value2='two',
    )
    assert html == ('<pre class="highlight">' '<del>- one</del>\n' '<ins>+ two</ins>' '</pre>')

    html = generate_ndiff(
        value1='aaa\nccc\nddd\n',
        value2='aaa\nbbb\nccc\n',
    )
    assert html == ('<pre class="highlight">' '  aaa\n' '<ins>+ bbb</ins>\n' '  ccc\n' '<del>- ddd</del>' '</pre>')


def test_generate_dmp_diff():
    html = generate_dmp_diff(
        value1='one',
        value2='two',
    )
    assert html == (
        '<pre class="highlight">'
        '<span class="diff-line diff-del diff-ins"><del>one</del><ins>two</ins></span>\n'
        '</pre>'
    )

    html = generate_dmp_diff(
        value1='aaa\nccc\nddd\n',
        value2='aaa\nbbb\nccc\n',
    )
    assert html == (
        '<pre class="highlight">aaa\n'
        '<span class="diff-line diff-del"><del>ccc</del></span>\n'
        '<span class="diff-line diff-del diff-ins"><del>ddd</del><ins>bbb</ins></span>\n'
        '<span class="diff-line diff-ins"><ins>ccc</ins></span>\n'
        '</pre>'
    )


def test_generate_dmp_diff_no_cleanup():
    """
    Test diffs created by google "diff-match-patch" without cleanup
    """
    html = generate_dmp_diff(value1='one', value2='two', cleanup=None)
    assert html == (
        '<pre class="highlight">'
        '<span class="diff-line diff-del diff-ins"><ins>tw</ins>o<del>ne</del></span>\n'
        '</pre>'
    )

    html = generate_dmp_diff(value1='aaa\nccc\nddd\n', value2='aaa\nbbb\nccc\n', cleanup=None)
    assert html == (
        '<pre class="highlight">aaa\n'
        '<span class="diff-line diff-ins"><ins>bbb</ins></span>\n'
        'ccc\n'
        '<span class="diff-line diff-del"><del>ddd</del></span>\n'
        '</pre>'
    )


def test_generate_dmp_diff_efficiency():
    """
    Test diffs created by google "diff-match-patch" with "efficiency" cleanup
    """
    html = generate_dmp_diff(value1='one', value2='two', cleanup=EFFICIENCY)
    assert html == (
        '<pre class="highlight">'
        '<span class="diff-line diff-del diff-ins"><ins>tw</ins>o<del>ne</del></span>\n'
        '</pre>'
    )

    html = generate_dmp_diff(value1='aaa\nccc\nddd\n', value2='aaa\nbbb\nccc\n', cleanup=EFFICIENCY)
    assert html == (
        '<pre class="highlight">aaa\n'
        '<span class="diff-line diff-ins"><ins>bbb</ins></span>\n'
        'ccc\n'
        '<span class="diff-line diff-del"><del>ddd</del></span>\n'
        '</pre>'
    )


def test_generate_dmp_diff_semantic():
    """
    Test diffs created by google "diff-match-patch" with "semantic" cleanup
    """
    html = generate_dmp_diff(value1='xxx1xxx\nX', value2='xxx2xxx\nX', cleanup=SEMANTIC)
    assert html == (
        '<pre class="highlight">'
        '<span class="diff-line diff-del diff-ins">xxx<del>1</del><ins>2</ins>xxx</span>\n'
        'X\n'
        '</pre>'
    )

    html = generate_dmp_diff(value1='one', value2='two', cleanup=SEMANTIC)
    assert html == (
        '<pre class="highlight">'
        '<span class="diff-line diff-del diff-ins"><del>one</del><ins>two</ins></span>\n'
        '</pre>'
    )

    html = generate_dmp_diff(value1='aaa\nccc\nddd\n', value2='aaa\nbbb\nccc\n', cleanup=SEMANTIC)
    assert html == (
        '<pre class="highlight">aaa\n'
        '<span class="diff-line diff-del"><del>ccc</del></span>\n'
        '<span class="diff-line diff-del diff-ins"><del>ddd</del><ins>bbb</ins></span>\n'
        '<span class="diff-line diff-ins"><ins>ccc</ins></span>\n'
        '</pre>'
    )


def test_html_diff():
    # small values -> ndiff
    html = html_diff(
        value1='one',
        value2='two',
    )
    assert html == '<pre class="highlight"><del>- one</del>\n<ins>+ two</ins></pre>'

    # big values -> Google diff-match-patch
    html = html_diff(
        value1='more than 20 Characters or?',
        value2='More than 20 characters, or?',
    )
    assert html == (
        '<pre class="highlight">'
        '<span class="diff-line diff-del diff-ins">'
        '<del>m</del><ins>M</ins>ore than 20 <del>C</del><ins>c</ins>haracters<ins>,</ins> or?'
        '</span>\n'
        '</pre>'
    )


def test_diff2lines():
    assert list(
        diff2lines(
            [
                (DIFF_EQUAL, 'equal\ntext'),
                (DIFF_DELETE, 'deleted\n'),
                (DIFF_INSERT, 'added\ntext'),
            ]
        )
    ) == [
        [(DIFF_EQUAL, 'equal')],
        [(DIFF_EQUAL, 'text'), (DIFF_DELETE, 'deleted')],
        [(DIFF_INSERT, 'added')],
        [(DIFF_INSERT, 'text')],
    ]

    # html escaping
    assert list(
        diff2lines(
            [
                (DIFF_EQUAL, '<equal>\ntext'),
                (DIFF_DELETE, '&deleted\n'),
                (DIFF_INSERT, 'added\ntext'),
            ]
        )
    ) == [
        [(DIFF_EQUAL, '&lt;equal&gt;')],
        [(DIFF_EQUAL, 'text'), (DIFF_DELETE, '&amp;deleted')],
        [(DIFF_INSERT, 'added')],
        [(DIFF_INSERT, 'text')],
    ]

    # \r\n line feeds
    assert list(
        diff2lines(
            [
                (DIFF_EQUAL, 'equal\r\ntext'),
                (DIFF_DELETE, 'deleted\r\n'),
                (DIFF_INSERT, 'added\r\ntext'),
            ]
        )
    ) == [
        [(DIFF_EQUAL, 'equal')],
        [(DIFF_EQUAL, 'text'), (DIFF_DELETE, 'deleted')],
        [(DIFF_INSERT, 'added')],
        [(DIFF_INSERT, 'text')],
    ]

    # Whitespace is retained
    assert list(
        diff2lines(
            [
                (DIFF_EQUAL, 'equal\ntext   '),
                (DIFF_DELETE, 'deleted\n'),
                (DIFF_INSERT, 'added\n   text'),
            ]
        )
    ) == [
        [(DIFF_EQUAL, 'equal')],
        [(DIFF_EQUAL, 'text   '), (DIFF_DELETE, 'deleted')],
        [(DIFF_INSERT, 'added')],
        [(DIFF_INSERT, '   text')],
    ]


def test_lines2html():
    assert lines2html(
        [
            [(DIFF_EQUAL, 'equal')],
            [(DIFF_EQUAL, 'text'), (DIFF_DELETE, 'deleted'), (DIFF_DELETE, '')],
            [(DIFF_INSERT, 'added')],
            [(DIFF_INSERT, 'text'), (DIFF_DELETE, 'removed')],
        ]
    ) == (
        'equal\n'
        '<span class="diff-line diff-del">text<del>deleted</del><del>⏎</del></span>\n'
        '<span class="diff-line diff-ins"><ins>added</ins></span>\n'
        '<span class="diff-line diff-del diff-ins"><ins>text</ins><del>removed</del></span>\n'
    )



"""
    :copyleft: 2015-2016 by the django-reversion-compare team, see AUTHORS for more details.
    :created: 2015 by JensDiemer.de
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""

from pathlib import Path

from django_tools.management.commands.run_testserver import Command as RunServerCommand

import reversion_compare_project
from reversion_compare_project.utils.fixtures import Fixtures


BASE_PATH = Path(reversion_compare_project.__file__).parent


class Command(RunServerCommand):
    def post_setup(self, **options) -> None:
        # insert all unittest data into database:
        Fixtures(verbose=True).create_all()



from bx_django_utils.test_utils.html_assertion import HtmlAssertionMixin
from django.apps import apps
from django.contrib.auth.models import User
from django.test import TestCase
from model_bakery import baker
from reversion import get_registered_models


class AdminAnonymousTests(HtmlAssertionMixin, TestCase):
    """
    Anonymous will be redirected to the login page.
    """

    def test_login_en(self):
        response = self.client.get("/en/admin/", HTTP_ACCEPT_LANGUAGE="en")
        self.assertRedirects(response, expected_url="/en/admin/login/?next=/en/admin/")

    def test_login_de(self):
        response = self.client.get("/de/admin/", HTTP_ACCEPT_LANGUAGE="de")
        self.assertRedirects(response, expected_url="/de/admin/login/?next=/de/admin/")


class AdminLoggedinTests(HtmlAssertionMixin, TestCase):
    """
    Some basics test with the django admin
    """

    @classmethod
    def setUpTestData(cls):
        cls.superuser = baker.make(User, username='superuser', is_staff=True, is_active=True, is_superuser=True)
        cls.staffuser = baker.make(User, username='staff_test_user', is_staff=True, is_active=True, is_superuser=False)

    def test_admin_login(self):
        response = self.client.get('/admin/')
        self.assertRedirects(response, expected_url='/en/admin/', fetch_redirect_response=False)
        response = self.client.get('/en/admin/')
        self.assertRedirects(response, expected_url='/en/admin/login/?next=/en/admin/', fetch_redirect_response=False)

        self.client.force_login(self.staffuser)
        response = self.client.get('/en/admin/')
        self.assertEqual(response.status_code, 200, response)
        self.assert_html_parts(response, parts=('<strong>staff_test_user</strong>',))

    def test_model_registering(self):
        test_app_config = apps.get_app_config(app_label='reversion_compare_project')
        models = test_app_config.get_models(include_auto_created=False, include_swapped=False)
        default_registered = len(list(get_registered_models()))
        self.assertEqual(default_registered, len(tuple(models)))

    def test_staff_admin_index(self):
        self.client.force_login(self.staffuser)

        response = self.client.get("/en/admin/", HTTP_ACCEPT_LANGUAGE="en")
        self.assert_html_parts(
            response,
            parts=(
                "<title>Reversion Compare Test | Site administration</title>",
                "<h1>Site administration</h1>",
                "<strong>staff_test_user</strong>",
                "<p>You don’t have permission to view or edit anything.</p>",
            ),
        )
        self.assertTemplateUsed(response, template_name="admin/index.html")

    def test_superuser_admin_index(self):
        self.client.force_login(self.superuser)
        response = self.client.get("/en/admin/", HTTP_ACCEPT_LANGUAGE="en")
        self.assert_html_parts(
            response,
            parts=(
                "reversion_compare",
                "<strong>superuser</strong>",
                "Site administration",
                "/admin/auth/group/add/",
                "/admin/auth/user/add/",
            ),
        )
        self.assertTemplateUsed(response, template_name="admin/index.html")



"""
    models
    ~~~~~~

    All example models would be used for django-reversion-compare unittests, too.

    :copyleft: 2012-2022 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from django.conf import settings
from django.core.validators import validate_comma_separated_integer_list
from django.db import models
from django_countries.fields import CountryField
from django_tools.file_storage.file_system_storage import OverwriteFileSystemStorage
from reversion import revisions


class SimpleModel(models.Model):
    text = models.CharField(max_length=255)

    def __str__(self):
        return f"SimpleModel pk: {self.pk!r} text: {self.text!r}"


class MigrationModel(models.Model):
    """
    Test model for "incompatible version data" fallback.
    See:
    https://github.com/jedie/django-reversion-compare/issues/148

    The model will be filled in migrations!
    """

    info = models.CharField(
        max_length=255, help_text='This field never changed and contains info about migration/version'
    )
    # number_then_text = models.CharField(
    #     max_length=32,
    #     help_text='migrate between CharField -> IntegerField -> CharField'
    # )
    number_then_text = models.IntegerField(help_text='migrate between CharField -> IntegerField -> CharField')

    # text = models.TextField(
    #     help_text='TextField -> CharField(max_length=128) -> CharField(max_length=64)'
    # )
    text = models.CharField(
        max_length=128, help_text='TextField -> CharField(max_length=128) -> CharField(max_length=64)'
    )
    # text = models.CharField(
    #     max_length=64,
    #     help_text='TextField -> CharField(max_length=128) -> CharField(max_length=64)'
    # )

    def __str__(self):
        return f"MigrationModel pk: {self.pk!r} text: {self.text!r}"


# ------------------------------------------------------------------------------


"""
models with relationships

Factory & Car would be only registered in admin.py
so no relation data would be stored

Person & Pet would be registered here with the follow information, so that
related data would be also stored in django-reversion

see "Advanced model registration" here:
    https://github.com/etianen/django-reversion/wiki/Low-level-API
"""


class Building(models.Model):
    address = models.CharField(max_length=128)

    def __str__(self):
        return self.address


class Factory(Building):
    name = models.CharField(max_length=128)

    def __str__(self):
        return self.name


class Car(models.Model):
    name = models.CharField(max_length=128)
    manufacturer = models.ForeignKey(Factory, related_name="cars", on_delete=models.CASCADE)
    supplier = models.ManyToManyField(Factory, related_name="suppliers", blank=True)

    def __str__(self):
        return (
            f"{self.name} from {self.manufacturer}" f" supplier(s): {', '.join(s.name for s in self.supplier.all())}"
        )


class Pet(models.Model):
    name = models.CharField(max_length=100)

    def __str__(self):
        return self.name


class Person(models.Model):
    name = models.CharField(max_length=100)
    pets = models.ManyToManyField(Pet, blank=True)
    # If you work someone, its at a build, but maybe not a factory!
    workplace = models.ForeignKey(Building, related_name="workers", null=True, on_delete=models.CASCADE)

    def __str__(self):
        return self.name


class Identity(models.Model):
    id_numer = models.CharField(max_length=100)
    person = models.OneToOneField(Person, related_name="_identity", on_delete=models.CASCADE)

    def __str__(self):
        return self.id_numer


revisions.register(Person, follow=["pets"])
revisions.register(Pet)


def get_filepath():
    return settings.MEDIA_ROOT


class VariantModel(models.Model):
    """
    This model should contain all variants of all existing types,
    without the related fields.

    TODO: Add reversion_compare_project for all variants!
    """

    TEST_CHOICES = (("a", "alpha"), ("b", "bravo"))
    boolean = models.BooleanField(default=True)
    null_boolean = models.BooleanField(null=True)

    char = models.CharField(max_length=1, blank=True, null=True)
    choices_char = models.CharField(max_length=1, blank=True, null=True, choices=TEST_CHOICES)
    text = models.TextField(blank=True, null=True)
    # skip: models.SlugField()

    integer = models.IntegerField(blank=True, null=True)
    integers = models.CharField(
        validators=[validate_comma_separated_integer_list], max_length=64, blank=True, null=True
    )
    positive_integer = models.PositiveIntegerField(blank=True, null=True)
    big_integer = models.BigIntegerField(blank=True, null=True)
    # skip:
    # models.PositiveSmallIntegerField()
    # models.SmallIntegerField()

    time = models.TimeField(blank=True, null=True)
    date = models.DateField(blank=True, null=True)
    datetime = models.DateTimeField(blank=True, null=True)

    decimal = models.DecimalField(max_digits=5, decimal_places=3, blank=True, null=True)
    float = models.FloatField(blank=True, null=True)

    email = models.EmailField(blank=True, null=True)
    url = models.URLField(blank=True, null=True)

    file_field = models.FileField(storage=OverwriteFileSystemStorage(create_backups=False), blank=True, null=True)

    filepath = models.FilePathField(path=get_filepath, blank=True, null=True)

    ip_address = models.GenericIPAddressField(blank=True, null=True)

    def __str__(self):
        return f"VariantModel instance pk: {self.pk:d}"


class CustomModel(models.Model):
    """Model which uses a custom version manager."""

    text = models.TextField()


class TemplateField(models.Model):
    """VersionAdmin in django-reversion has field compare_template that is str
    Model used for check correct handling this case
    Should be used ForeignKey for calling mixins.CompareMixin._get_compare_func
    """

    # some field for easy creating revisions
    text = models.CharField(max_length=20)
    template = models.ForeignKey(Person, blank=True, null=True, on_delete=models.CASCADE)


class CountryFieldTestModel(models.Model):
    # Test with model fields from https://github.com/SmileyChris/django-countries
    one_country = CountryField()
    multiple_countries = CountryField(multiple=True)



#!/usr/bin/env python

"""
    django-reversion-compare unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    :copyleft: 2022 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from reversion.models import Version

from reversion_compare_project.utils.fixtures import Fixtures
from reversion_compare_project.utils.test_cases import BaseTestCase


class CountryFieldTestModelTest(BaseTestCase):
    def setUp(self):
        super().setUp()

        # fixtures = Fixtures(verbose=False)
        fixtures = Fixtures(verbose=True)
        self.instance = fixtures.create_CountryFieldTestModel_data()

        queryset = Version.objects.get_for_object(self.instance)
        self.version_ids = queryset.values_list("pk", flat=True)

    def test_diff(self):
        response = self.client.get(
            f"/en/admin/reversion_compare_project/countryfieldtestmodel/{self.instance.pk}/history/compare/",
            data={"version_id2": self.version_ids[0], "version_id1": self.version_ids[1]},
        )
        self.assertEqual(response.status_code, 200, response)
        self.assertTemplateUsed(response, 'reversion-compare/compare.html')
        self.assert_html_parts(
            response,
            parts=(
                "<h3>one country</h3>",
                """
                <div class="module">
                    <pre class="highlight"><del>- Germany</del>
                    <ins>+ United Kingdom</ins></pre>
                </div>
                """,
                "<h3>multiple countries</h3>",
                # TODO: Enhance Django-Countries diff:
                """
                <span class="diff-line diff-del diff-ins">
                [&#x27;Austria&#x27;, &#x27;<del>Germany</del>
                <ins>Switzerland&#x27;, &#x27;Germany&#x27;, &#x27;United Kingdom</ins>&#x27;]
                </span>
                """,
            ),
        )



from django.conf import settings
from django.conf.urls import include, static
from django.conf.urls.i18n import i18n_patterns
from django.contrib import admin
from django.urls import path
from django.views.generic import RedirectView

from reversion_compare_project.views import SimpleModelHistoryCompareView


admin.autodiscover()


urlpatterns = i18n_patterns(
    path("test_view/<path:pk>/", SimpleModelHistoryCompareView.as_view(), name='test_view'),
    #
    # The Django admin with redirect:
    path('', RedirectView.as_view(url="/admin/")),
    path("admin/", admin.site.urls),
)


if settings.SERVE_FILES:
    urlpatterns += static.static(settings.STATIC_URL, document_root=settings.STATIC_ROOT)


if settings.DEBUG:
    import debug_toolbar

    urlpatterns = [
        path('__debug__/', include(debug_toolbar.urls)),
    ] + urlpatterns



#!/usr/bin/env python

"""
    django-reversion-compare unittests
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    I used the setup from reversion_compare_test_project !

    TODO:
        * models.OneToOneField()
        * models.IntegerField()

    :copyleft: 2012-2016 by the django-reversion-compare team, see AUTHORS for more details.
    :license: GNU GPL v3 or above, see LICENSE for more details.
"""


from django.urls import reverse
from reversion import create_revision
from reversion.models import Revision, Version

from reversion_compare_project.models import CustomModel
from reversion_compare_project.utils.fixtures import Fixtures
from reversion_compare_project.utils.test_cases import BaseTestCase


class CustomModelTest(BaseTestCase):
    """Test a model which uses a custom reversion manager."""

    def setUp(self):
        super().setUp()
        fixtures = Fixtures(verbose=False)
        self.item = fixtures.create_CustomModel_data()

    def test_initial_state(self):
        """ "Test initial data creation and model registration."""
        self.assertEqual(CustomModel.objects.count(), 1)
        self.assertEqual(Version.objects.get_for_object(self.item).count(), 1)
        self.assertEqual(Revision.objects.all().count(), 1)

    def test_text_diff(self):
        """ "Generate a new revision and check for a correctly generated diff."""
        with create_revision():
            self.item.text = "version two"
            self.item.save()
        queryset = Version.objects.get_for_object(self.item)
        version_ids = queryset.values_list("pk", flat=True)
        self.assertEqual(len(version_ids), 2)
        url_name = f"admin:{CustomModel._meta.app_label}_{CustomModel._meta.model_name}_compare"
        diff_url = reverse(url_name, args=(self.item.pk,))
        data = {"version_id2": version_ids[0], "version_id1": version_ids[1]}
        response = self.client.get(diff_url, data=data)
        self.assertContains(response, "<del>- version one</del>")
        self.assertContains(response, "<ins>+ version two</ins>")

    def test_version_selection(self):
        """Generate two revisions and view the version history selection."""
        with create_revision():
            self.item.text = "version two"
            self.item.save()
        with create_revision():
            self.item.text = "version three"
            self.item.save()
        queryset = Version.objects.get_for_object(self.item)
        version_ids = queryset.values_list("pk", flat=True)
        self.assertEqual(len(version_ids), 3)
        url_name = f"admin:{CustomModel._meta.app_label}_{CustomModel._meta.model_name}_history"
        history_url = reverse(url_name, args=(self.item.pk,))
        response = self.client.get(history_url)
        self.assertContainsHtml(
            response,
            '<input type="submit" value="compare">',
            f'<input type="radio" name="version_id1" value="{version_ids[0]:d}" style="visibility:hidden" />',
            f'<input type="radio" name="version_id2" value="{version_ids[0]:d}" checked="checked" />',
            f'<input type="radio" name="version_id1" value="{version_ids[1]:d}" checked="checked" />',
            f'<input type="radio" name="version_id2" value="{version_ids[1]:d}" />',
            f'<input type="radio" name="version_id1" value="{version_ids[2]:d}" />',
            f'<input type="radio" name="version_id2" value="{version_ids[2]:d}" />',
        )
